package com.ruoyi.ucp.service.impl;

import cn.hutool.core.date.DateTime;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.support.SFunction;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ruoyi.common.annotation.DataSource;
import com.ruoyi.common.enums.DataSourceType;
import com.ruoyi.ucp.entity.PointInfo;
import com.ruoyi.ucp.entity.StationInfoDay;
import com.ruoyi.ucp.mapper.StationInfoDayMapper;
import com.ruoyi.ucp.service.IStationInfoDayService;
import com.ruoyi.ucp.util.DatePart;
import com.ruoyi.ucp.util.SqlUtil;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Collection;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author gfhd
 * @since 2024-03-05
 */
@Service
@DataSource(DataSourceType.SLAVE)
public class StationInfoDayServiceImpl extends ServiceImpl<StationInfoDayMapper, StationInfoDay> implements IStationInfoDayService {

    public Map<String, StationInfoDay> getByEntity(PointInfo pointInfo) {
        LambdaQueryWrapper<StationInfoDay> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(pointInfo.getStationId() != null, StationInfoDay::getStationId, pointInfo.getStationId());
        //wrapper.eq(pointInfo.getRecordDate() != null, StationInfoDay::getRecordDate, pointInfo.getRecordDate());
        List<StationInfoDay> stationInfoDays = baseMapper.selectList(wrapper);

        return stationInfoDays.stream().collect(Collectors.toMap(StationInfoDay::getStationId, Function.identity()));
    }

    @Override
    @Transactional
    public boolean saveOrUpdateBatch(Collection<StationInfoDay> entityList) {
        return super.saveOrUpdateBatch(entityList);
    }

    @Override
    public StationInfoDay getOne(Wrapper<StationInfoDay> queryWrapper) {
        return super.getOne(queryWrapper);
    }

    @Override
    public List<StationInfoDay> list(Wrapper<StationInfoDay> queryWrapper) {
        return super.list(queryWrapper);
    }

    public Map<String, StationInfoDay> getDatasByPartSelect(Date date, DatePart datePart, String selects, SFunction<StationInfoDay,String> groupId) {
        DateTime[] beginEnd = SqlUtil.getBeginEnd(date, datePart);
        QueryWrapper<StationInfoDay> wrapper = new QueryWrapper<>();
        wrapper.select(selects);
        wrapper.lambda().between(StationInfoDay::getRecordDate, beginEnd[0], beginEnd[1])
                .select(groupId).groupBy(groupId);
        List<StationInfoDay> days = super.list(wrapper);
        return days.stream().collect(Collectors.toMap(groupId, Function.identity()));
    }
    public List<StationInfoDay> getDatasByPartSelects(Date date, DatePart datePart, String selects, String groupId) {
        DateTime[] beginEnd = SqlUtil.getBeginEnd(date, datePart);
        QueryWrapper<StationInfoDay> wrapper = new QueryWrapper<>();
        wrapper.select(selects + groupId).groupBy(groupId).between("record_date", beginEnd[0], beginEnd[1]);
        return super.list(wrapper);
    }
}
